#ifdef GL_ES
precision highp float;
#endif

// Input per material
uniform highp mat4 u_modelViewProjectionMatrix;
uniform mediump vec4 u_materialSpecular; // Fog start+end values are stored in .r and .g, divided by 10 to give range [0..2550]
uniform highp float u_time;

// Input per vertex
attribute highp vec4 a_position;
attribute mediump vec2 a_texCoord;
attribute lowp vec4 a_color;

// Output per vertex
varying mediump vec2 v_texCoord;
varying mediump float v_fogFactor;
varying lowp vec4 v_color;

// Returns 1 for (0 < t < startTime) and (endTime < t < 1)
// and a smooth dip to 0 between startTime and endTime
float calcBlinkAmount(float startTime, float endTime, float time) {

	float midTime = (startTime + endTime) * 0.5;
	float pre = 1.0 - smoothstep(startTime, midTime, time);
	float post = smoothstep(midTime, endTime, time);
	
	return (pre + post);
}

// Shader entry point
void main() {

	// Transform position
	vec4 pos = u_modelViewProjectionMatrix * a_position;

	// Calculate fog factor from values packed into specular material colour
	float fogStart = float(u_materialSpecular.r) * 255.0 * 10.0;
	float fogEnd = float(u_materialSpecular.g) * 255.0 * 10.0;
	v_fogFactor = clamp((pos.z - fogStart) / (fogEnd - fogStart), 0.0, 1.0);

	// Pass on the UVs unchanged
	v_texCoord = a_texCoord;
	
	// Blinking lights use mesh vertex colours if present
	// R : apply fade (0 = yes, 1 = no, as 1 is the more likely default value)
	// G : phase offset (so all lights don't blink exactly in sequence)
	float dontBlink = a_color.r;
	float phaseOffset = a_color.g;

	// Generate a periodic time value in the range [0..1]
	float t = fract((u_time * 1.0) + phaseOffset);

	// Calculate a blink amount based on the time and whether we want to blink at all
	float varyingBlinkAmount = calcBlinkAmount(0.65, 1.0, t);
	float blinkAmount = mix(varyingBlinkAmount, 1.0, dontBlink);

	// Generate a colour based on this blink amount
	v_color = vec4(blinkAmount, blinkAmount, blinkAmount, dontBlink);
	
	gl_Position = pos;
}